/**
 * Copyright (C) 2011 Ovea <dev@ovea.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ovea.markup;

import com.ovea.markup.util.Resource;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Locale;

/**
 * @author Mathieu Carbou (mathieu.carbou@gmail.com)
 */
public final class FileSystemTemplateResolver implements TemplateResolver {

    private final File webDir;

    public FileSystemTemplateResolver(File baseDir) {
        try {
            this.webDir = baseDir.getCanonicalFile();
        } catch (IOException e) {
            throw new IllegalArgumentException(e.getMessage(), e);
        }
    }

    @Override
    public Resource resolve(String path, Locale locale) throws TemplateResolverException {
        if (path == null) {
            throw new TemplateResolverException("Empty path");
        }
        path = path.toLowerCase();
        StringBuilder templateName = new StringBuilder(path);
        if (templateName.charAt(0) == '/')
            templateName.deleteCharAt(0);
        int pos = templateName.lastIndexOf(".");
        if (pos == -1) {
            throw new TemplateResolverException("Illegal path: extension needed");
        }
        String[] locales = new String[]{"_" + locale, "_" + locale.getLanguage(), ""};
        for (int i = 0, prev = 2; i < locales.length; i++, prev = (2 + i) % 3) {
            templateName.replace(pos, pos + locales[prev].length(), locales[i]);
            File file = new File(webDir, templateName.toString());
            if (file.exists()) {
                try {
                    if (!file.getCanonicalFile().getAbsolutePath().startsWith(webDir.getAbsolutePath())) {
                        throw new TemplateResolverException("Web template folder " + webDir + " cannot serve template " + path);
                    }
                    return Resource.from(file.toURI().toURL());
                } catch (MalformedURLException e) {
                    throw new TemplateResolverException("Error getting URL from file " + file + " : " + e.getMessage(), e);
                } catch (IOException e) {
                    throw new TemplateResolverException("Error accessing file path " + file + " : " + e.getMessage(), e);
                }
            }
        }
        throw new TemplateResolverException("Cannot resolve template for path " + path);
    }
}
